#!/bin/bash

# abort whenever an error occurs
set -e

if [ -z "$NEXTCLOUD_VERSION" ]
then
    echo "No nextcloud version set, using version in file"
    export NEXTCLOUD_VERSION=$(cat NEXTCLOUD_VERSION | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
    echo "Found version: ${NEXTCLOUD_VERSION}"
fi

if [ -z "$NEXTCLOUD_VERSION" ]
then
    echo "Error: No nextcloud version set!"
    exit 1
fi

# Cleanup old client dirs
rm -rf client

echo "Checking out nextcloud client version: ${NEXTCLOUD_VERSION}"

# Clone the desktop client code
git clone --recursive https://github.com/nextcloud/desktop.git client
cd client

git checkout ${NEXTCLOUD_VERSION}
git submodule update --recursive

echo "Applying TransIP patches"

# apply all patches
for file in ../patches/*; do echo "Applying patch ${file}" && patch -p1 < ${file}; done

cd ..

echo "Copying NSIS installer files for windows build"
# Because the windows installer build script does not use the custom theme directory
cp -R transiptheme/installer/nsi client/admin/win/

echo "Adapting theme"

mkdir -p stacktheme/theme
# Copy nextcloud theme files
cp -R client/theme/* stacktheme/theme
# Overwrite the STACK specific theme files
cp -R transiptheme/* stacktheme/

echo "Ready for build ID: ${BUILDID}"